from funsecret import read_secret
from fundb.sqlalchemy import BaseTable as BaseTable2
from fundb.sqlalchemy import create_engine


class BaseTable(BaseTable2):
    def __init__(self, *args, **kwargs):
        uri = read_secret(cate1="funcoin", cate2="dataset", cate3="cron", cate4="uri")
        super(BaseTable, self).__init__(engine=create_engine(uri), *args, **kwargs)



import ccxt
from fundrive import AlipanDrive
from fundrive.core.table import DriveTable
from funsecret import read_secret

from funcoin.coins.table.load import LoadTask


def download_daily():
    exchange = ccxt.binance(  # noqa: F821
        {
            "apiKey": read_secret("coin", "binance", "api_key"),
            "secret": read_secret("coin", "binance", "secret_key"),
        }
    )

    drive = AlipanDrive()
    drive.login()
    table = DriveTable(table_fid="66bf1a293f124e5a2d2841eeb0fbb21b70ddb0ee", drive=drive)
    task = LoadTask(table=table, exchange=exchange)
    task.run()



import csv
import json
import logging
import time

import ccxt
import pandas as pd
from ccxt.base.exchange import Exchange
from tqdm import tqdm

logger = logging.getLogger("funcoin")
unix_month = 2678400000
one_hour = 3600 * 1000


class BaseLoader:
    def __init__(self, unix_start, unix_end, *args, **kwargs):
        self.unix_start = unix_start
        self.unix_end = unix_end
        self.cache_data = []

    def _open(self, *args, **kwargs):
        pass

    def _write(self, data_list):
        pass

    def _close(self, *args, **kwargs):
        pass

    def _load_symbols(self, *args, **kwargs):
        pass

    def _load_symbol(self, symbol, pbr=None, *args, **kwargs):
        pass

    def load_symbols(self, *args, **kwargs):
        self._open(*args, **kwargs)
        self._load_symbols(*args, **kwargs)
        self._close(*args, **kwargs)

    def load_symbol(self, symbol, pbr=None, *args, **kwargs):
        self._open(*args, **kwargs)
        self._load_symbol(symbol=symbol, pbr=pbr, *args, **kwargs)
        self._close(*args, **kwargs)

    def write_data(self, data_list, cache=True):
        self.cache_data.extend(data_list)
        if cache and len(self.cache_data) < 10000:
            return
        if len(self.cache_data) == 0:
            return
        df = pd.DataFrame(self.cache_data)
        df = df[(df["timestamp"] >= self.unix_start) & (df["timestamp"] <= self.unix_end)]
        self._write(json.loads(df.to_json(orient="records")))
        self.cache_data.clear()

    def __enter__(self):
        self._handle = self
        self._open()
        return self._handle

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.cache_data is not None:
            self.write_data([], cache=False)
        self._close()
        return True


class CSVLoader(BaseLoader):
    def __init__(self, csv_path, fieldnames, *args, **kwargs):
        self.csv_path = csv_path
        super().__init__(*args, **kwargs)
        self.csv_file = open(self.csv_path, mode="w")
        self.csv_writer = csv.DictWriter(self.csv_file, delimiter=",", fieldnames=fieldnames)
        self.csv_writer.writeheader()

    def _write(self, data_list):
        self.csv_writer.writerows(data_list)
        self.csv_file.flush()

    def _close(self, *args, **kwargs):
        self.csv_file.close()


class CCXTBaseLoader(CSVLoader):
    def __init__(self, exchange: Exchange, *args, **kwargs):
        self.exchange = exchange
        super().__init__(*args, **kwargs)
        self.exchange.load_markets()

    def _load_symbols(self, *args, **kwargs):
        pbr = tqdm(self.exchange.symbols)
        for sym in pbr:
            if ":" not in sym:
                pbr.set_description(sym)
                self._load_symbol(sym, pbr, *args, **kwargs)
        self.write_data([], False)


class KlineLoder(CCXTBaseLoader):
    def __init__(self, *args, timeframe="1m", **kwargs):
        super(KlineLoder, self).__init__(
            fieldnames=["symbol", "timestamp", "open", "close", "low", "high", "vol"],
            *args,
            **kwargs,
        )
        self.timeframe = timeframe

    def _load_symbol(self, symbol, pbr=None, *args, **kwargs):
        unix_temp = self.unix_start
        for _ in range(1000):
            if unix_temp >= self.unix_end:
                break
            try:
                result = self.exchange.fetch_ohlcv(symbol, self.timeframe, unix_temp, limit=100)
                result = self.exchange.sort_by(result, 0)
                if len(result) == 0:
                    break
                unix_temp = result[-1][0]
                df = pd.DataFrame(result, columns=["timestamp", "open", "close", "low", "high", "vol"])
                df["symbol"] = symbol
                self.write_data(json.loads(df.to_json(orient="records")))
                time.sleep(int(self.exchange.rateLimit / 1000))
            except Exception as e:
                print(type(e).__name__, str(e))
                self.exchange.sleep(10000)


class TradeLoader(CCXTBaseLoader):
    def __init__(self, *args, **kwargs):
        super(TradeLoader, self).__init__(
            fieldnames=["symbol", "id", "timestamp", "side", "price", "amount"],
            *args,
            **kwargs,
        )

    def _load_symbol(self, symbol, pbr=None, *args, **kwargs):
        unix_temp = self.unix_start
        previous_trade_id = None
        for _ in range(10000):
            pbr.set_description(f"{symbol}-{unix_temp}")
            if unix_temp >= self.unix_end:
                break
            try:
                trades = self.exchange.fetch_trades(symbol, unix_temp, limit=1000)
                if len(trades) == 0:
                    unix_temp += one_hour
                    continue
                last_trade = trades[-1]
                if previous_trade_id == last_trade["id"]:
                    unix_temp += one_hour
                    continue
                unix_temp = last_trade["timestamp"]
                previous_trade_id = last_trade["id"]

                result = [
                    {
                        "symbol": trade["symbol"],
                        "id": trade["id"].replace("\n", ""),
                        "timestamp": int(trade["timestamp"]),
                        "side": trade["side"][0],
                        "price": trade["price"],
                        "amount": trade["amount"],
                    }
                    for trade in trades
                ]

                self.write_data(result)
                time.sleep(int(self.exchange.rateLimit / 1000))
            except ccxt.NetworkError as e:
                print(type(e).__name__, str(e))
                self.exchange.sleep(1000)



from funcoin.okex.client.base import BaseClient
from funcoin.okex.consts import GET, POST


class AssetClient(BaseClient):

    def __init__(self, *args, **kwargs):
        super(AssetClient, self).__init__(*args, **kwargs)

    def get_deposit_address(self, ccy):
        uri = '/api/v5/asset/deposit-address'
        params = {'ccy': ccy}
        return self._request_with_params(GET, uri, params)

    def get_balances(self, ccy=None):
        uri = '/api/v5/asset/balances'
        params = {'ccy': ccy}
        return self._request_with_params(GET, uri, params)

    def funds_transfer(self, ccy, amt, froms, to, type='0', subAcct=None, instId=None, toInstId=None):
        uri = '/api/v5/asset/transfer'
        params = {'ccy': ccy, 'amt': amt, 'from': froms, 'to': to, 'type': type, 'subAcct': subAcct, 'instId': instId,
                  'toInstId': toInstId}
        return self._request_with_params(POST, uri, params)

    def coin_withdraw(self, ccy, amt, dest, toAddr, pwd, fee):
        uri = '/api/v5/asset/withdrawal'
        params = {'ccy': ccy, 'amt': amt, 'dest': dest, 'toAddr': toAddr, 'pwd': pwd, 'fee': fee}
        return self._request_with_params(POST, uri, params)

    def get_deposit_history(self, ccy=None, state=None, after=None, before=None, limit=None):
        uri = '/api/v5/asset/deposit-history'
        params = {'ccy': ccy, 'state': state, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_withdrawal_history(self, ccy=None, state=None, after=None, before=None, limit=None):
        uri = '/api/v5/asset/withdrawal-history'
        params = {'ccy': ccy, 'state': state, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_currency(self):
        uri = '/api/v5/asset/currencies'
        return self._request_without_params(GET, uri)

    def purchase_redempt(self, ccy, amt, side):
        uri = '/api/v5/asset/purchase_redempt'
        params = {'ccy': ccy, 'amt': amt, 'side': side}
        return self._request_with_params(POST, uri, params)

    def get_bills(self, ccy=None, type=None, after=None, before=None, limit=None):
        uri = '/api/v5/asset/bills'
        params = {'ccy': ccy,         'type': type,         'after': after,         'before': before,         'limit': limit}
        return self._request_with_params(GET, uri, params)

        



import calendar
import datetime
import os
import time
from datetime import timedelta

from fundrive.lanzou import LanZouCloud
from fundrive.tables import SqliteTable


class AutoSaveDetail(SqliteTable):
    def __init__(
        self, db_path=None, save_freq="d", fid=-1, ms=False, save_key=None, save_with_clear=True, *args, **kwargs
    ):
        if db_path is None:
            db_path = os.path.abspath(os.path.dirname(__file__)) + "/data/coin.db"

        super(AutoSaveDetail, self).__init__(db_path=db_path, *args, **kwargs)

        self.ms = ms
        self.fid = fid
        self.save_key = save_key
        self.save_freq = save_freq
        self.save_with_clear = save_with_clear
        self.last_finish_save_unix = datetime.datetime.now()
        self.last_temp_save_unix = datetime.datetime.now()

        if self.save_freq in ("d", "D", "Day", "day"):
            self.save_freq = "d"
        elif self.save_freq in ("m", "M", "mon", "Mon", "month", "Month"):
            self.save_freq = "m"
        elif self.save_freq in ("week", "Week"):
            self.save_freq = "w"
        elif self.save_freq in ("Y", "y", "year", "Year"):
            self.save_freq = "y"
        else:
            self.save_freq = "d"

        self.path_root = os.path.join(os.path.dirname(self.db_path), self.table_name)
        if not os.path.exists(self.path_root):
            os.makedirs(self.path_root)

        self.downer = LanZouCloud()
        self.downer.ignore_limits()
        self.downer.login_by_cookie()

    def auto_save(self):
        """
        数据保存，分两种，
        一种是数据已经准备完成，上传云端,每天凌晨1点以后保存一次
        另一种是保存中间结果，每隔3个小时保存一次
        """
        time_unix = datetime.datetime.now()

        is_finish_save = False

        if self.save_freq in ("d", "D", "Day", "day"):
            is_finish_save = self.last_finish_save_unix.day != time_unix.day and time_unix.hour >= 1
        elif self.save_freq in ("m", "M", "mon", "Mon", "month", "Month"):
            is_finish_save = self.last_finish_save_unix.month != time_unix.month and time_unix.hour >= 1
        elif self.save_freq in ("m", "M", "mon", "Mon", "month", "Month"):
            is_finish_save = self.last_finish_save_unix.year != time_unix.year and time_unix.hour >= 1

        is_temp_save = abs(self.last_temp_save_unix.hour - time_unix.hour) >= 3
        if is_finish_save:
            self._save(now=self.last_finish_save_unix, pop=self.save_with_clear)
            self.last_finish_save_unix = time_unix
            self.downer.sync_files(self.path_root, folder_id=self.fid, overwrite=True, remove_local=True)
        elif is_temp_save:
            self._save(pop=False)
            self.last_temp_save_unix = time_unix

    def _save(self, pop=None, now=None):
        now = now or datetime.datetime.now()

        name = self.table_name
        if self.save_freq in ("d", "D", "Day", "day"):
            fmt = "%Y%m%d"
        elif self.save_freq in ("m", "M", "mon", "Mon", "month", "Month"):
            fmt = "%Y%m"
        elif self.save_freq in ("week", "Week"):
            fmt = "%Y%U"
        elif self.save_freq in ("Y", "y", "year", "Year"):
            fmt = "%Y"
        else:
            fmt = "%Y%m%d"
        if self.save_key is not None:
            start, end = self.between_unix_ms(now)
            condition = "{key}>={start} and {key}<={end}".format(key=self.save_key, start=start, end=end)
            condition2 = "{key}<={end}".format(key=self.save_key, end=end)
        else:
            condition = None
            condition2 = None

        path = "{}/{}-{}.csv".format(self.path_root, name, self.last_finish_save_unix.strftime(fmt))

        self.to_csv(condition=condition, path=path, index=None)
        if pop:
            self.delete(condition2)

    def between_unix_ms(self, now):
        now = now or datetime.datetime.now()

        this_day_start = now.date()
        this_day_end = this_day_start + timedelta(days=1)
        this_week_start = (now - timedelta(days=now.weekday())).date()
        this_week_end = this_week_start + timedelta(days=7)
        this_month_start = datetime.datetime(now.year, now.month, 1).date()
        this_month_end = this_month_start + timedelta(days=calendar.monthrange(now.year, now.month)[1])
        this_year_start = datetime.datetime(now.year, 1, 1).date()
        this_year_end = datetime.datetime(now.year + 1, 1, 1).date()

        this_day_start = self.date2unix(this_day_start.timetuple())
        this_day_end = self.date2unix(this_day_end.timetuple()) - 1
        this_week_start = self.date2unix(this_week_start.timetuple())
        this_week_end = self.date2unix(this_week_end.timetuple()) - 1
        this_month_start = self.date2unix(this_month_start.timetuple())
        this_month_end = self.date2unix(this_month_end.timetuple()) - 1
        this_year_start = self.date2unix(this_year_start.timetuple())
        this_year_end = self.date2unix(this_year_end.timetuple()) - 1

        if self.save_freq in ("d", "D", "Day", "day"):
            return this_day_start, this_day_end
        elif self.save_freq in ("m", "M", "mon", "Mon", "month", "Month"):
            return this_month_start, this_month_end
        elif self.save_freq in ("week", "Week"):
            return this_week_start, this_week_end
        elif self.save_freq in ("Y", "y", "year", "Year"):
            return this_year_start, this_year_end
        else:
            return this_day_start, this_day_end

    def date2unix(self, t):
        if self.ms:
            return round(time.mktime(t) * 1000)
        return round(time.mktime(t))


class TradeDetail(AutoSaveDetail):
    def __init__(self, table_name="trade_detail", db_path=None, *args, **kwargs):
        if db_path is None:
            db_path = os.path.abspath(os.path.dirname(__file__)) + "/data/coin.db"

        super(TradeDetail, self).__init__(
            db_path=db_path, ms=True, fid=3359101, save_key="ts", table_name=table_name, *args, **kwargs
        )
        self.columns = ["trade_id", "amount", "price", "ts", "direction"]

    def create(self):
        self.execute(
            """
            create table if not exists {} (
               trade_id       BIGINT         
              ,amount         FLOAT 
              ,price          FLOAT
              ,ts             BIGINT
              ,direction      VARCHAR(5)
              ,primary key (trade_id)           
              )
            """.format(
                self.table_name
            )
        )


class SymbolInfo(AutoSaveDetail):
    def __init__(self, table_name="symbol_info", db_path=None, *args, **kwargs):
        if db_path is None:
            db_path = os.path.abspath(os.path.dirname(__file__)) + "/data/coin.db"

        super(SymbolInfo, self).__init__(
            db_path=db_path,
            table_name=table_name,
            save_freq="y",
            fid=3360754,
            save_key="",
            save_with_clear=False,
            *args,
            **kwargs
        )
        self.columns = [
            "symbol",
            "base_currency",
            "quote_currency",
            "price_precision",
            "amount_precision",
            "symbol_partition",
            "state",
            "value_precision",
            "min_order_amt",
            "max_order_amt",
            "limit_order_min_order_amt",
            "limit_order_max_order_amt",
            "sell_market_min_order_amt",
            "sell_market_max_order_amt",
            "buy_market_max_order_value",
            "min_order_value",
            "max_order_value",
            "leverage_ratio",
            "underlying",
            "mgmt_fee_rate",
            "charge_time",
            "rebal_time",
            "rebal_threshold",
            "init_nav",
            "api_trading",
        ]

    def create(self):
        self.execute(
            """
                create table if not exists {} (
                symbol		                    VARCHAR	-- 交易对
                ,base_currency                  VARCHAR -- 交易对中的基础币种
                ,quote_currency		            VARCHAR	-- 交易对中的报价币种
                ,price_precision		        integer	-- 交易对报价的精度（小数点后位数）
                ,amount_precision		        integer	-- 交易对基础币种计数精度（小数点后位数）
                ,symbol_partition		        VARCHAR	-- 交易区，可能值: [main，innovation]
                ,`state`		                VARCHAR	-- 交易对状态；可能值: [online，offline,suspend] online _ 已上线；offline _ 交易对已下线，不可交易；suspend -- 交易暂停；pre_online _ 即将上线
                ,value_precision		        integer	-- 交易对交易金额的精度（小数点后位数）
                ,min_order_amt		            float	-- 交易对限价单最小下单量 ，以基础币种为单位（即将废弃）
                ,max_order_amt		            float	-- 交易对限价单最大下单量 ，以基础币种为单位（即将废弃）
                ,limit_order_min_order_amt		float	-- 交易对限价单最小下单量 ，以基础币种为单位（NEW）
                ,limit_order_max_order_amt		float	-- 交易对限价单最大下单量 ，以基础币种为单位（NEW）
                ,sell_market_min_order_amt		float	-- 交易对市价卖单最小下单量，以基础币种为单位（NEW）
                ,sell_market_max_order_amt		float	-- 交易对市价卖单最大下单量，以基础币种为单位（NEW）
                ,buy_market_max_order_value		float	-- 交易对市价买单最大下单金额，以计价币种为单位（NEW）
                ,min_order_value		        float	-- 交易对限价单和市价买单最小下单金额 ，以计价币种为单位
                ,max_order_value		        float	-- 交易对限价单和市价买单最大下单金额 ，以折算后的USDT为单位（NEW）
                ,leverage_ratio		            float	-- 交易对杠杆最大倍数(仅对逐仓杠杆交易对、全仓杠杆交易对、杠杆ETP交易对有效）
                ,underlying		                VARCHAR	-- 标的交易代码 (仅对杠杆ETP交易对有效)
                ,mgmt_fee_rate		            float	-- 持仓管理费费率 (仅对杠杆ETP交易对有效)
                ,charge_time		            VARCHAR	-- 持仓管理费收取时间 (24小时制，GMT+8，格式：HH:MM:SS，仅对杠杆ETP交易对有效)
                ,rebal_time		                VARCHAR	-- 每日调仓时间 (24小时制，GMT+8，格式：HH:MM:SS，仅对杠杆ETP交易对有效)
                ,rebal_threshold		        float	-- 临时调仓阈值 (以实际杠杆率计，仅对杠杆ETP交易对有效)
                ,init_nav		                float	-- 初始净值（仅对杠杆ETP交易对有效）
                ,api_trading		            VARCHAR	-- API交易使能标记（有效值：enabled, disabled）
                ,PRIMARY KEY (symbol)
                )
                """.format(
                self.table_name
            )
        )


class KlineDetail(AutoSaveDetail):
    def __init__(self, table_name="kline", db_path=None, *args, **kwargs):
        if db_path is None:
            db_path = os.path.abspath(os.path.dirname(__file__)) + "/data/coin.db"

        super(KlineDetail, self).__init__(db_path=db_path, save_key="id", table_name=table_name, *args, **kwargs)
        self.columns = ["symbol", "id", "amount", "count", "open", "close", "low", "high", "vol"]

    def create(self):
        self.execute(
            """
            create table if not exists {} (
               symbol	VARCHAR	-- 交易对
              ,id	    bigint	-- 调整为新加坡时间的时间戳，单位秒，并以此作为此K线柱的id
              ,amount	float	-- 以基础币种计量的交易量
              ,`count`	integer	-- 交易次数
              ,`open`   float	-- 本阶段开盘价
              ,`close`	float	-- 本阶段收盘价
              ,low	    float	-- 本阶段最低价
              ,high	    float	-- 本阶段最高价
              ,vol	    float	-- 以报价币种计量的交易量
              ,primary key (symbol,id)           
              )
            """.format(
                self.table_name
            )
        )


class Kline1MinDetail(KlineDetail):
    def __init__(self, table_name="kline_1min", db_path=None, *args, **kwargs):
        super(Kline1MinDetail, self).__init__(table_name=table_name, fid=3359096, db_path=db_path, *args, **kwargs)


class Kline5MinDetail(KlineDetail):
    def __init__(self, table_name="kline_5min", db_path=None, *args, **kwargs):
        super(Kline5MinDetail, self).__init__(table_name=table_name, fid=3359092, db_path=db_path, *args, **kwargs)


class Kline15MinDetail(KlineDetail):
    def __init__(self, table_name="kline_15min", db_path=None, *args, **kwargs):
        super(Kline15MinDetail, self).__init__(table_name=table_name, fid=3359095, db_path=db_path, *args, **kwargs)


class Kline30MinDetail(KlineDetail):
    def __init__(self, table_name="kline_30min", db_path=None, *args, **kwargs):
        super(Kline30MinDetail, self).__init__(
            table_name=table_name, db_path=db_path, fid=3359097, save_freq="m", *args, **kwargs
        )


class Kline60MinDetail(KlineDetail):
    def __init__(self, table_name="kline_60min", db_path=None, *args, **kwargs):
        super(Kline60MinDetail, self).__init__(
            table_name=table_name, fid=3360759, db_path=db_path, save_freq="m", *args, **kwargs
        )


class Kline4HourDetail(KlineDetail):
    def __init__(self, table_name="kline_4hour", db_path=None, *args, **kwargs):
        super(Kline4HourDetail, self).__init__(
            table_name=table_name, save_freq="m", fid=3359094, db_path=db_path, *args, **kwargs
        )


class Kline1DayDetail(KlineDetail):
    def __init__(self, table_name="kline_1day", db_path=None, *args, **kwargs):
        super(Kline1DayDetail, self).__init__(
            table_name=table_name, db_path=db_path, fid=3359093, save_freq="y", *args, **kwargs
        )


class Kline1MonDetail(KlineDetail):
    def __init__(self, table_name="kline_1mon", db_path=None, *args, **kwargs):
        super(Kline1MonDetail, self).__init__(
            table_name=table_name, db_path=db_path, save_freq="y", fid=3359099, save_with_clear=False, *args, **kwargs
        )


class Kline1WeekDetail(KlineDetail):
    def __init__(self, table_name="kline_1week", db_path=None, *args, **kwargs):
        super(Kline1WeekDetail, self).__init__(
            table_name=table_name, db_path=db_path, save_freq="y", fid=3359098, save_with_clear=False, *args, **kwargs
        )


class Kline1YearDetail(KlineDetail):
    def __init__(self, table_name="kline_1year", db_path=None, *args, **kwargs):
        super(Kline1YearDetail, self).__init__(
            table_name=table_name, db_path=db_path, save_freq="y", fid=3359091, save_with_clear=False, *args, **kwargs
        )



from funcoin.huobi.utils.input_checker import *


class MarginClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs

    def post_transfer_in_margin(self, symbol: 'str', currency: 'str', amount: 'float') -> int:
        """
        Transfer asset from spot account to margin account.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param currency: The currency of transfer. (mandatory)
        :param amount: The amount of transfer. (mandatory)
        :return:
        """
        check_symbol(symbol)
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "symbol": symbol,
            "currency": currency,
            "amount": amount
        }

        from huobi.service.margin.post_transfer_in_margin import PostTransferInMarginService
        return PostTransferInMarginService(params).request(**self.__kwargs)

    def post_transfer_out_margin(self, symbol: 'str', currency: 'str', amount: 'float') -> int:
        """
        Transfer asset from margin account to spot account.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param currency: The currency of transfer. (mandatory)
        :param amount: The amount of transfer. (mandatory)
        :return:
        """
        check_symbol(symbol)
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "symbol": symbol,
            "currency": currency,
            "amount": amount
        }

        from huobi.service.margin.post_transfer_out_margin import PostTransferOutMarginService
        return PostTransferOutMarginService(params).request(**self.__kwargs)

    def get_margin_account_balance(self, symbol: 'str') -> list:
        """
        Get the Balance of the Margin Loan Account.

        :param symbol: The currency, like "btc". (mandatory)
        :return: The margin loan account detail list.
        """
        check_symbol(symbol)

        params = {
            "symbol": symbol
        }

        from huobi.service.margin.get_margin_account_balance import GetMarginAccountBalanceService
        return GetMarginAccountBalanceService(params).request(**self.__kwargs)

    def post_create_margin_order(self, symbol: 'str', currency: 'str', amount: 'float') -> int:
        """
        Submit a request to borrow with margin account.

        :param symbol: The trading symbol to borrow margin, e.g. "btcusdt", "bccbtc". (mandatory)
        :param currency: The currency to borrow,like "btc". (mandatory)
        :param amount: The amount of currency to borrow. (mandatory)
        :return: The margin order id.
        """
        check_symbol(symbol)
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "symbol": symbol,
            "currency": currency,
            "amount": amount
        }

        from huobi.service.margin.post_create_margin_order import PostCreateMarginOrderService
        return PostCreateMarginOrderService(params).request(**self.__kwargs)

    def post_repay_margin_order(self, loan_id: 'int', amount: 'float') -> int:
        """
        Get the margin loan records.

        :param load_id: The previously returned order id when loan order was created. (mandatory)
        :param amount: The amount of currency to repay. (mandatory)
        :return: The margin order id.
        """
        check_should_not_none(loan_id, "loan_id")
        check_should_not_none(amount, "amount")

        params = {
            "loan_id": loan_id,
            "amount": amount
        }

        from huobi.service.margin.post_repay_margin_order import PostRepayMarginOrderService
        return PostRepayMarginOrderService(params).request(**self.__kwargs)

    def get_margin_loan_orders(self, symbol: 'str', start_date: 'str' = None, end_date: 'str' = None,
                               states: 'LoanOrderState' = None, from_id: 'int' = None,
                               size: 'int' = None, direction: 'QueryDirection' = None) -> list:
        """
        Get the margin loan records.

        :param symbol: The symbol, like "btcusdt" (mandatory).
        :param start_date: The search starts date in format yyyy-mm-dd. (optional).
        :param end_date: The search end date in format yyyy-mm-dd.(optional, can be null).
        :param states: The loan order states, it could be created, accrual, cleared or invalid. (optional)
        :param from_id: Search order id to begin with. (optional)
        :param size: The number of orders to return.. (optional)
        :param direction: The query direction, prev or next. (optional)
        :return: The list of the margin loan records.
        """

        check_symbol(symbol)
        start_date = format_date(start_date, "start_date")
        end_date = format_date(end_date, "end_date")

        params = {
            "symbol": symbol,
            "start-date": start_date,
            "end-date": end_date,
            "states": states,
            "from": from_id,
            "size": size,
            "direct": direction
        }

        from huobi.service.margin.get_margin_loan_orders import GetMarginLoanOrdersService
        return GetMarginLoanOrdersService(params).request(**self.__kwargs)

    def get_margin_loan_info(self, symbols: 'str' = None) -> list:
        """
        The request of get margin loan info, can return currency loan info list.

        :param symbols: The symbol, like "btcusdt,htusdt". (optional)
        :return: The cross margin loan info.
        """

        check_symbol(symbols)
        params = {
            "symbols": symbols
        }

        from huobi.service.margin.get_margin_loan_info import GetMarginLoanInfoService
        return GetMarginLoanInfoService(params).request(**self.__kwargs)

    def get_cross_margin_loan_info(self) -> list:
        """
        The request of currency loan info list.

        :return: The cross margin loan info list.
        """
        params = {}

        from huobi.service.margin.get_cross_margin_loan_info import GetCrossMarginLoanInfoService
        return GetCrossMarginLoanInfoService(params).request(**self.__kwargs)

    def post_cross_margin_transfer_in(self, currency: 'str', amount: 'float') -> int:
        """
        transfer currency to cross account.

        :param currency: currency name (mandatory)
        :param amount: transfer amount (mandatory)
        :return: return transfer id.
        """
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "amount": amount,
            "currency": currency
        }

        from huobi.service.margin.post_cross_margin_transfer_in import PostCrossMarginTransferInService
        return PostCrossMarginTransferInService(params).request(**self.__kwargs)

    def post_cross_margin_transfer_out(self, currency: 'str', amount: 'float') -> int:
        """
        transfer currency to cross account.

        :param currency: currency name (mandatory)
        :param amount: transfer amount (mandatory)
        :return: return transfer id.
        """
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "amount": amount,
            "currency": currency
        }

        from huobi.service.margin.post_cross_margin_transfer_out import PostCrossMarginTransferOutService
        return PostCrossMarginTransferOutService(params).request(**self.__kwargs)

    def post_cross_margin_create_loan_orders(self, currency: 'str', amount: 'float') -> int:
        """
        create cross margin loan orders

        :param currency: currency name (mandatory)
        :param amount: transfer amount (mandatory)
        :return: return order id.
        """

        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "amount": amount,
            "currency": currency
        }

        from huobi.service.margin.post_cross_margin_create_loan_orders import PostCrossMarginCreateLoanOrdersService
        return PostCrossMarginCreateLoanOrdersService(params).request(**self.__kwargs)

    def post_cross_margin_loan_order_repay(self, order_id: 'str', amount: 'float'):
        """
        repay cross margin loan orders

        :param order_id: order_id for loan (mandatory)
        :param amount: transfer amount (mandatory)
        :return: return order id.
        """

        check_should_not_none(order_id, "order-id")
        check_should_not_none(amount, "amount")

        params = {
            "amount": amount,
            "order-id": order_id
        }

        from huobi.service.margin.post_cross_margin_loan_order_repay import PostCrossMarginLoanOrderRepayService
        return PostCrossMarginLoanOrderRepayService(params).request(**self.__kwargs)

    def get_cross_margin_loan_orders(self, currency: 'str' = None, state: 'str' = None,
                                     start_date: 'str' = None, end_date: 'str' = None,
                                     from_id: 'int' = None, size: 'int' = None, direct: 'str' = None,
                                     sub_uid: 'int' = None) -> list:
        """
        get cross margin loan orders

        :return: return list.
        """
        params = {
            "currency": currency,
            "state": state,
            "start-date": start_date,
            "end-date": end_date,
            "from": from_id,
            "size": size,
            "direct": direct,
            "sub-uid": sub_uid
        }

        from huobi.service.margin.get_cross_margin_loan_orders import GetCrossMarginLoanOrdersService
        return GetCrossMarginLoanOrdersService(params).request(**self.__kwargs)

    def get_cross_margin_account_balance(self, sub_uid: 'int' = None):
        """
        get cross margin account balance

        :return: cross-margin account.
        """
        params = {
            "sub-uid": sub_uid
        }

        from huobi.service.margin.get_cross_margin_account_balance import GetCrossMarginAccountBalanceService
        return GetCrossMarginAccountBalanceService(params).request(**self.__kwargs)

    def post_general_repay_loan(self, account_id: 'str', currency: 'str', amount: 'float',
                                transact_id: 'str' = None) -> list:
        """
        Repay Margin Loan（Cross）.

        :param account_id: repayment account ID .(mandatory)
        :param currency: repayment currency. (mandatory).
        :param amount: repayment amount.(mandatory).
        :param transact_id: loan transaction ID. (optional)
        """
        check_should_not_none(account_id, "account_id")
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")

        params = {
            "accountId": account_id,
            "currency": currency,
            "amount": amount,
            "transact_id": transact_id
        }

        from huobi.service.margin.post_general_repay_loan import PostGeneralRepayLoanService
        return PostGeneralRepayLoanService(params).request(**self.__kwargs)

    def get_general_repayment_loan_records(self, repay_id: 'str' = None, account_id: 'str' = None,
                                           currency: 'str' = None, start_time: 'int' = None, end_time: 'int' = None,
                                           sort: 'str' = None, limit: 'int' = None, from_id: 'int' = None) -> list:

        """
         Get Repayment Record Reference（Cross）.

         :param repay_id: repayment transaction ID. (optional)
         :param account_id: account ID (default value: all accounts) (optional).
         :param currency: borrowing/lending currency (default value: all currencies). (optional)
         :param start_time: start time (unix time in millisecond; range: [(endTime – x D), endTime]; default value: (endTime – x D) (optional)
         :param end_time: end time (unix time in millisecond；range: [(present time – y D), present time]; default value: present time) (optional)
         :param sort: sort direction (virtual value: asc, desc; default value: desc) (optional)
         :param limit: max return items per page (range: [1,100]; default value: 50) (optional)
         :param from_id: search original ID (only available when searching for the next page) (optional)
         """

        params = {

        }
        if repay_id is not None:
            params['repayId'] = repay_id

        if account_id is not None:
            params['accountId'] = account_id

        if account_id is not None:
            params['currency'] = currency

        if start_time is not None:
            params['startTime'] = start_time

        if end_time is not None:
            params['endTime'] = end_time

        if sort is not None:
            params['sort'] = sort

        if limit is not None:
            params['limit'] = limit

        if from_id is not None:
            params['fromId'] = from_id

        from huobi.service.margin.get_general_repayment_loan_records import GetGeneralRepaymentLoanRecordsService
        return GetGeneralRepaymentLoanRecordsService(params).request(**self.__kwargs)

        return



from datetime import datetime, timedelta

from ccxt import binance
from funsecret import read_secret

from funcoin.coins.base.file import DataFileProperty
from funcoin.task import BaseTask


class LoadTask(BaseTask):
    table_name = "load"

    def __init__(self, *args, **kwargs):
        self.path_root = (
            read_secret(cate1="funcoin", cate2="path", cate3="local", cate4="path_root") or "~/workspace/tmp"
        )
        super(LoadTask, self).__init__(*args, **kwargs)

    @staticmethod
    def parse_day(ds, days=0):
        ds = ds
        first = datetime.strptime(ds, "%Y-%m-%d") - timedelta(days=days)
        last = first + timedelta(days=1) - timedelta(seconds=1)
        return first, last

    @staticmethod
    def parse_week(ds, weeks=0):
        ds = ds
        first = datetime.strptime(ds, "%Y-%m-%d") + timedelta(weeks=weeks)
        first = first - timedelta(days=first.weekday())
        first = datetime(first.year, first.month, first.day)
        last = first + timedelta(weeks=1) - timedelta(seconds=1)
        return first, last


class LoadKlineDailyTask(LoadTask):
    def refresh(self, ds):
        start, end = self.parse_day(ds)
        file_pro = DataFileProperty(exchange=binance(), path=self.path_root)
        file_pro.file_format = "%Y%m%d"
        file_pro.change_data_type("kline")
        file_pro.change_timeframe("1m")
        file_pro.change_freq("daily")
        file_pro.load_daily(start, end)


class LoadKlineWeeklyTask(LoadTask):
    def refresh(self, ds):
        start, end = self.parse_week(ds, weeks=-1)
        file_pro = DataFileProperty(exchange=binance(), path=self.path_root)
        file_pro.file_format = "%Y%m%d"
        file_pro.change_data_type("kline")
        file_pro.change_timeframe("1m")
        file_pro.change_freq("weekly")
        print((start, end))
        file_pro.load_merge(start, end)


class LoadTradeDailyTask(LoadTask):
    def refresh(self, ds):
        start, end = self.parse_day(ds)
        file_pro = DataFileProperty(exchange=binance(), path=self.path_root)
        file_pro.file_format = "%Y%m%d"
        file_pro.change_data_type("trade")
        file_pro.change_timeframe("detail")
        file_pro.change_freq("daily")
        file_pro.load_daily(start, end)


# LoadKlineWeeklyTask().refresh('2021-9-14')



import logging
import os
from datetime import datetime, timedelta

import ccxt
from fundrive.core.table import DriveTable
from funfile.compress import tarfile

from funcoin.coins.base.loader import BaseLoader, KlineLoder, TradeLoader

logging.getLogger("fundrive").setLevel(logging.INFO)
logger = logging.getLogger("funcoin")
logger.setLevel(logging.INFO)


class FileProperty:
    def __init__(self, exchange_name, data_type="kline", timeframe="1m"):
        self.data_type = data_type
        self.timeframe = timeframe
        self.exchange_name = exchange_name

        self.freq = None
        self.end_date = None
        self.start_date = None
        self.par_format = None
        self.file_format = None

    def daily(self, ds):
        self.freq = "daily"
        self.file_format = self.file_format or "%Y%m%d"
        self.par_format = self.par_format or "%Y%m"
        self.start_date = datetime.strptime(ds, "%Y%m%d")
        self.end_date = self.start_date + timedelta(days=1)
        return self

    @property
    def partition(self):
        return self.start_date.strftime(self.par_format)

    @property
    def filename_prefix(self):
        return f"{self.exchange_name}_{self.data_type}_{self.freq}_{self.timeframe}-{self.start_date.strftime(self.file_format)}"

    @property
    def file_path_csv(self):
        return f"{self.filename_prefix}.csv"

    @property
    def file_path_tar(self):
        return f"{self.filename_prefix}.tar"


class LoadTask:
    def __init__(
        self,
        table: DriveTable,
        exchange: ccxt.Exchange,
    ):
        self.table = table
        self.exchange = exchange

    def download(self, loader: BaseLoader, file_pro: FileProperty) -> bool:
        logger.info(f"download for {file_pro.file_path_tar}")
        # 下载
        loader.load_symbols()
        # 压缩
        with tarfile.open(file_pro.file_path_tar, "w|xz") as tar:
            tar.add(file_pro.file_path_csv)
        self.table.upload(file=file_pro.file_path_tar, partition=file_pro.partition, overwrite=True)

        # 删除
        if os.path.exists(file_pro.file_path_csv):
            os.remove(file_pro.file_path_csv)
        if os.path.exists(file_pro.file_path_tar):
            os.remove(file_pro.file_path_tar)
        return True

    def download_kline(self, file_pro: FileProperty) -> bool:
        loader = KlineLoder(
            self.exchange,
            unix_start=int(file_pro.start_date.timestamp() * 1000),
            unix_end=int(file_pro.end_date.timestamp() * 1000),
            csv_path=file_pro.file_path_csv,
            timeframe=file_pro.timeframe,
        )
        return self.download(loader, file_pro)

    def download_trade(self, file_pro: FileProperty) -> bool:
        loader = TradeLoader(
            self.exchange,
            unix_start=int(file_pro.start_date.timestamp() * 1000),
            unix_end=int(file_pro.end_date.timestamp() * 1000),
            csv_path=file_pro.file_path_csv,
            timeframe=file_pro.timeframe,
        )
        return self.download(loader, file_pro)

    def run(self, days=365):
        self.table.update_partition_dict()
        self.table.update_partition_meta()

        start_day = datetime.now() - timedelta(days=-1)
        file_pro = FileProperty(self.exchange.name.lower()).daily(start_day.strftime("%Y%m%d"))
        exists_data = dict([file["name"], file] for file in self.table.partition_meta())

        for i in range(days):
            start_day += timedelta(days=-1)
            file_pro.daily(start_day.strftime("%Y%m%d"))
            if file_pro.file_path_tar in exists_data.keys():
                logger.info(f"{file_pro.file_path_tar} exists, skip.")
                continue
            self.download_kline(file_pro)



from darkbuild.tool.fastapi import add_api_routes, api_route
from funcoin.database.base import create_all, create_session
from funcoin.database.connect import RedisConnect
from funcoin.okex.client.const import account_api
from funcoin.okex.database.client import OkexClientAccountBalance


class AccountAccount(RedisConnect):
    def __init__(self, prefix="/account", cache_prefix='okex_account_balance', *args, **kwargs):
        super(AccountAccount, self).__init__(prefix=prefix, cache_prefix=cache_prefix, *args, **kwargs)
        add_api_routes(self)
        self.session = create_session()
        create_all()

    @api_route('/update', description="update market tickers")
    def update_value(self, suffix=""):
        try:
            data = account_api.get_account().data[0]['details']
            self.put_value(self.get_key(suffix=suffix), data)
            for detail in data:
                print(detail)
                create_all()
                self.session.merge(OkexClientAccountBalance(**detail))
            self.session.commit()
            create_all()
            return {"success": len(data)}
        except Exception as e:
            self.session.rollback()
            return {"failed": str(e)}

    def get_value(self, suffix=""):
        return super(AccountAccount, self).get_value(suffix=suffix)



import os

from ccxt import binance
from fastapi import APIRouter
from darkbuild.tool.fastapi import add_api_routes, api_route
from funcoin.coins.base.file import DataFileProperty
from funsecret import read_secret

path_root = read_secret(cate1="funcoin", cate2="path", cate3="local", cate4="path_root") or "~/workspace/tmp"


class DownloadServer(APIRouter):
    def __init__(self, prefix="/download", *args, **kwargs):
        super(DownloadServer, self).__init__(prefix=prefix, *args, **kwargs)
        self.runing = False
        add_api_routes(self)

    @api_route("/kline", description="get value")
    def download_kline(self, freq="daily", timeframe="1m"):
        if self.runing:
            return {"status": "false", "msg": "is running"}
        self.runing = True

        os.rmdir(f"{path_root}/funcoin")
        file_pro = DataFileProperty(exchange=binance(), path=path_root)
        file_pro.change_data_type("kline")
        file_pro.change_timeframe(timeframe=timeframe)
        file_pro.change_freq(freq)
        file_pro.load(total=400)
        file_pro.change_freq("weekly")
        file_pro.load(total=55)

        self.runing = False

    @api_route("/trade", description="get value")
    def download_trade(self, freq="daily"):
        if self.runing:
            return {"status": "false", "msg": "is running"}
        self.runing = True

        os.rmdir(f"{path_root}/funcoin")
        file_pro = DataFileProperty(exchange=binance(), path=path_root)
        file_pro.change_data_type("trade")
        file_pro.change_timeframe("detail")
        file_pro.change_freq(freq)
        file_pro.load(total=400)

        self.runing = False

    @api_route("/auto", description="get value")
    def download_auto(self, freq="daily"):
        if self.runing:
            return {"status": "false", "msg": "is running"}
        self.runing = True

        file_pro = DataFileProperty(exchange=binance(), path=path_root)
        for i in range(1, 500):
            file_pro.change_data_type("kline")
            file_pro.change_timeframe("1m")
            file_pro.change_freq(freq)
            file_pro.load(total=i)
            file_pro.change_freq("weekly")
            file_pro.load(total=i // 7)

            file_pro.change_data_type("trade")
            file_pro.change_timeframe("detail")
            file_pro.change_freq(freq)
            file_pro.load(total=i)
        self.runing = False

    def clear(self):
        path = f"{path_root}/funcoin"
        if os.path.exists(path):
            os.rmdir(path)



"""

"""
import json
import zipfile
from datetime import datetime

import pandas as pd
from funcoin.huobi.dataset.core import (
    Kline1DayDetail,
    Kline1MinDetail,
    Kline4HourDetail,
    Kline5MinDetail,
    Kline15MinDetail,
    Kline30MinDetail,
    Kline60MinDetail,
    KlineDetail,
    TradeDetail,
)
from funcoin.huobi.history.utils import *
from funtool.log import log, logger
from funtool.path import list_file, removedirs
from tqdm import tqdm

_SWAP = "swap"
_SPOT = "spot"
_FUTURE = "future"
_OPTION = "option"
_LINEARSWAP = "linear-swap"
ALL_TYPES = ["future", "spot", "swap", "option", "linear-swap"]
ALL_DATA_TYPES = ["klines", "trades"]
ALL_PERIODS = ["1min", "5min", "15min", "30min", "60min", "4hour", "1day"]
ALL_FREQ = ["daily"]
PRE_URL = "https://futures.huobi.com/data"

logger = log()


class Response:
    def __init__(self, status: bool = True, data: dict = None):
        self.status = status or True
        self.data = data or {}

    def success(self):
        self.status = True

    def error(self):
        self.status = False

    def put(self, key, value):
        self.data[key] = value

    def get(self, key):
        return self.data.get(key)

    def extend(self, data: dict = None):
        if isinstance(data, dict):
            self.data.update(data)


class HistoryDownload:
    def __init__(
        self,
        pre_url=None,
        data_type: str = None,
        all_periods: list = None,
        _type: str = None,
        freq: str = None,
        start_date=None,
        end_date=None,
        all_symbols=None,
        download_dir=None,
    ):
        self.freq = freq or ALL_FREQ[0]
        self.type = _type or ALL_TYPES[0]
        self.pre_url = pre_url or PRE_URL
        self.data_type = data_type or ALL_DATA_TYPES[0]

        self.all_symbols = all_symbols
        self.download_dir = download_dir or "./data"
        self.all_periods = all_periods or ALL_PERIODS
        self.start_date = start_date or datetime(2017, 10, 27)
        self.end_date = end_date or datetime(2021, 7, 27)

    @staticmethod
    def _file_unzip(file_path) -> Response:
        """unzip zip file"""
        response = Response(True)
        zip_file = zipfile.ZipFile(file_path)
        file_dir = os.path.split(file_path)[0]
        for name in zip_file.namelist():
            zip_file.extract(name, path=file_dir)
            response.put("file_path", os.path.join(file_dir, name))
        zip_file.close()

        return response

    @staticmethod
    def _http_download(url: str, download_dir) -> Response:
        response = Response()
        try:
            if not os.path.exists(download_dir):
                os.makedirs(download_dir)
            if url is None:
                return Response(False, {"msg": "url is null"})
            data = requests.get(url, allow_redirects=True)
            file_name = os.path.basename(url)
            file_path = os.path.join(download_dir, file_name)
            if len(data.content) > 30:
                with open(file_path, "wb") as f:
                    f.write(data.content)
                response.put("file_path", file_path)
            else:
                response.error()
                response.put("file_path", "File not exists.")

        except Exception as e:
            response.put("download error", str(e))

        return response

    def init_parameters(self):
        print(self.type)
        if self.all_symbols is None:
            ok, all_symbols = False, []
            if self.type == _SPOT:
                ok, all_symbols = get_all_spot_symbols()
            elif self.type == _FUTURE:
                ok, all_symbols = get_all_future_symbols()
            elif self.type == _SWAP:
                ok, all_symbols = get_all_swap_symbols()
            elif self.type == _OPTION:
                ok, all_symbols = get_all_option_symbols()
            elif self.type == _LINEARSWAP:
                ok, all_symbols = get_all_linearswap_symbols()

            if not ok:
                logger.warning(all_symbols)

                return
            self.all_symbols = all_symbols

    def download_symbol_period_day(
        self, path_url, symbol, period, current, delete_zip=True, delete_check=True, *args, **kwargs
    ) -> Response:
        url = f"{path_url}/{symbol.upper()}-{period}-{current.year}-{current.month:02}-{current.day:02}"
        download_dir = os.path.join(self.download_dir, self.data_type, self.type, period)
        if not os.path.exists(download_dir):
            os.makedirs(download_dir)
        zip_file = f"{url}.zip"
        check_file = f"{url}.CHECKSUM"
        response1 = self._http_download(zip_file, download_dir)
        response2 = self._http_download(check_file, download_dir)
        response = Response(response1.status)

        response.extend(
            {
                "zip_url": zip_file,
                "zip_file": response1.get("file_path"),
                "check_url": check_file,
                "check_file": response2.get("file_path"),
            }
        )

        if response1.status:
            response3 = self._file_unzip(response1.get("file_path"))
            response3.put("file", response3.get("file_path"))
            if delete_zip:
                os.remove(response.get("zip_file"))
            if delete_check:
                os.remove(response.get("check_file"))

        return response

    def download_symbol_period(self, symbol, period, start_date=None, end_date=None, *args, **kwargs) -> tuple:
        if period in [
            "trades",
        ]:
            path_url = f"{self.pre_url}/{self.data_type}/{self.type}/{self.freq}/{symbol}"
        else:
            path_url = f"{self.pre_url}/{self.data_type}/{self.type}/{self.freq}/{symbol}/{period}"
        start_date = start_date or self.start_date
        end_date = end_date or self.end_date
        all_res = []
        interval = end_date - start_date + timedelta(days=1)
        if interval.days > 10:
            for index in tqdm(range(interval.days), desc=f"{symbol}"):
                current = start_date + timedelta(days=index)
                all_res.append(self.download_symbol_period_day(path_url, symbol, period, current, *args, **kwargs))
        else:
            for index in range(interval.days):
                current = start_date + timedelta(days=index)
                all_res.append(self.download_symbol_period_day(path_url, symbol, period, current, *args, **kwargs))
        all_oks = [response for response in all_res if response.status]
        all_errs = [response for response in all_res if not response.status]
        return all_oks, all_errs

    def download_symbols(self, all_symbols=None, all_periods=None, start_date=None, end_date=None, *args, **kwargs):
        """return date is: [start, end)"""
        self.init_parameters()
        all_symbols = all_symbols or self.all_symbols
        all_periods = all_periods or self.all_periods
        if len(all_symbols) > 10:
            for symbol in tqdm(all_symbols, desc="download symbols"):
                for period in all_periods:
                    all_oks, all_errs = self.download_symbol_period(
                        symbol, period, start_date, end_date, *args, **kwargs
                    )
                    logger.warning(f"success:{all_oks}")
                    logger.warning(f"failed:{all_errs}")
        else:
            for symbol in all_symbols:
                for period in all_periods:
                    all_oks, all_errs = self.download_symbol_period(
                        symbol, period, start_date, end_date, *args, **kwargs
                    )
                    logger.warning(f"success:{all_oks}")
                    logger.warning(f"failed:{all_errs}")
        logger.info("done")

    def download_klines(self, _type=None, *args, **kwargs):
        """return date is: [start, end)"""
        self.data_type = ALL_DATA_TYPES[0]
        self.type = _type or ALL_TYPES[0]
        self.download_symbols(*args, **kwargs)

    def download_trades(self, _type=None, *args, **kwargs):
        self.type = _type or ALL_TYPES[0]
        self.data_type = ALL_DATA_TYPES[1]
        self.all_periods = ["trades"]
        self.download_symbols(*args, **kwargs)


class SymbolHistory:
    def __init__(self, dir_path="/root/workspace/tmp/coin/", symbol="SHIBUSDT"):
        self.db_path = os.path.join(dir_path, "dbs", f"huobi-{symbol}.db")

        self.file_dir = os.path.join(dir_path, "files")

        self.symbol = symbol
        self.tradeDB = TradeDetail(db_path=self.db_path)
        self.kline_1min = Kline1MinDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.kline_5min = Kline5MinDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.kline_15min = Kline15MinDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.kline_30min = Kline30MinDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.kline_60min = Kline60MinDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.kline_4hour = Kline4HourDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.kline_1day = Kline1DayDetail(db_path=self.db_path, conn=self.tradeDB.conn)
        self.init()

    def init(self):
        self.tradeDB.create()
        self.kline_1min.create()
        self.kline_5min.create()
        self.kline_15min.create()
        self.kline_30min.create()
        self.kline_60min.create()
        self.kline_4hour.create()
        self.kline_1day.create()

    def insert_trades(self, start_date, end_date):
        pass

    def insert_kline(self, kline_db: KlineDetail, file_path):
        try:
            df = pd.read_csv(file_path, index_col=None, header=None)
            df.columns = ["id", "open", "close", "high", "low", "vol", "amount"]
            df["symbol"] = self.symbol
            kline_db.insert_list(json.loads(df.to_json(orient="records")))
        except Exception as e:
            logger.info(f"error file {file_path}")
            print(e)

    def insert_klines(self, start_date, end_date):
        self.insert_kline(
            self.kline_1min,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="1min"),
        )
        self.insert_kline(
            self.kline_5min,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="5min"),
        )
        self.insert_kline(
            self.kline_15min,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="15min"),
        )
        self.insert_kline(
            self.kline_30min,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="30min"),
        )
        self.insert_kline(
            self.kline_60min,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="60min"),
        )
        self.insert_kline(
            self.kline_4hour,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="4hour"),
        )
        self.insert_kline(
            self.kline_1day,
            load_symbol_all(symbol=self.symbol, start_date=start_date, end_date=end_date, period="1day"),
        )

    def insert_data(self, start_date, end_date):
        self.insert_trades(start_date, end_date)
        self.insert_klines(start_date, end_date)

        print(self.tradeDB.select("select count(1) as num from " + self.tradeDB.table_name))
        print("1min\t{}".format(self.kline_1min.select("select count(1) as num from " + self.kline_1min.table_name)))
        print("5min\t{}".format(self.kline_5min.select("select count(1) as num from " + self.kline_5min.table_name)))
        print("15min\t{}".format(self.kline_15min.select("select count(1) as num from " + self.kline_15min.table_name)))
        print("30min\t{}".format(self.kline_30min.select("select count(1) as num from " + self.kline_30min.table_name)))
        print("60min\t{}".format(self.kline_60min.select("select count(1) as num from " + self.kline_60min.table_name)))
        print("4hour\t{}".format(self.kline_4hour.select("select count(1) as num from " + self.kline_4hour.table_name)))
        print("1day\t{}".format(self.kline_1day.select("select count(1) as num from " + self.kline_1day.table_name)))

    def run(self, start_date=datetime(2021, 5, 25), end_date=datetime(2021, 6, 23)):
        self.insert_data(start_date, end_date)

    def test(self):
        import pandas as pd

        condition = "id>=1619400078439 and id<=1619452799999 limit 111"
        sql = "select * from {} where {}".format(self.kline_1day.table_name, condition)
        print(sql)
        result = pd.read_sql(sql=sql, con=self.kline_1day.conn)
        print(result.head(10))


def load_daily_all(
    period="1min",
    data_type=ALL_DATA_TYPES[0],
    _type=ALL_TYPES[0],
    date=datetime(2021, 5, 21),
    download_dir="/root/workspace/tmp/coin/daily",
    delete_file=True,
    overwrite=False,
):
    download_dir2 = os.path.join(download_dir, date.strftime("%Y%m%d"))
    history = HistoryDownload(
        all_symbols=None,
        data_type=data_type,
        all_periods=[period],
        _type=_type,
        start_date=date,
        end_date=date,
        download_dir=download_dir2,
    )
    file_path = os.path.join(download_dir, f'{history.data_type}_{history.type}_{period}-{date.strftime("%Y%m%d")}.csv')
    if os.path.exists(file_path) and not overwrite:
        return file_path
    if history.data_type == "trade":
        history.download_trades()
    else:
        history.download_klines()

    with open(file_path, "w") as new_file:
        for item in list_file(download_dir2, deep=10):
            symbol = os.path.basename(item).split("-")[0]
            for txt in open(item, "r"):
                txt.replace("\n", f"\n{symbol},")
                new_file.write(f"{symbol},{txt}")

    if delete_file:
        removedirs(download_dir2)
    return file_path


def load_symbol_all(
    symbol="SHIBUSDT",
    period="1min",
    _type=ALL_TYPES[0],
    data_type=ALL_DATA_TYPES[0],
    start_date=datetime(2021, 5, 1),
    end_date=datetime(2021, 5, 23),
    download_dir="/root/workspace/tmp/coin/symbol",
    delete_file=True,
):
    download_dir2 = os.path.join(download_dir, symbol)
    history = HistoryDownload(
        all_symbols=[symbol],
        data_type=data_type,
        all_periods=[period],
        start_date=start_date,
        end_date=end_date,
        _type=_type,
        download_dir=download_dir2,
    )
    history.download_klines()
    file_path = os.path.join(download_dir, f"{history.data_type}_{history.type}_{period}-{symbol}.csv")
    with open(file_path, "w") as new_file:
        for item in list_file(download_dir2, deep=10):
            for txt in open(item, "r"):
                new_file.write(txt)

    if delete_file:
        removedirs(download_dir2)
    return file_path


def load_symbol_all_to_db(symbol="SHIBUSDT", start_date=datetime(2021, 5, 1), end_date=datetime(2021, 5, 3)):
    history = SymbolHistory(symbol=symbol)
    history.run(start_date=start_date, end_date=end_date)
    return history.db_path
